@@ -1,6 +1,3 @@
-//go:build go1.18
-// +build go1.18
-
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License. See License.txt in the project root for license information.
 
@@ -22,6 +19,7 @@ import (
 	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
 	"github.com/Azure/azure-sdk-for-go/sdk/internal/mock"
 	"github.com/Azure/azure-sdk-for-go/sdk/internal/recording"
+	azcred "github.com/Azure/azure-sdk-for-go/sdk/internal/test/credential"
 	"github.com/Azure/azure-sdk-for-go/sdk/security/keyvault/azkeys"
 	"github.com/stretchr/testify/require"
 )
@@ -228,7 +226,7 @@ func TestDisableChallengeResourceVerification(t *testing.T) {
 				},
 				DisableChallengeResourceVerification: test.disableVerify,
 			}
-			client, err := azkeys.NewClient(vaultURL, &FakeCredential{}, options)
+			client, err := azkeys.NewClient(vaultURL, &azcred.Fake{}, options)
 			require.NoError(t, err)
 			pager := client.NewListKeyPropertiesPager(nil)
 			_, err = pager.NextPage(context.Background())
@@ -537,73 +535,72 @@ func TestRecoverDeletedKey(t *testing.T) {
 }
 
 func TestReleaseKey(t *testing.T) {
-	// Skipping managed HSM tests for now, service failure
-	//for _, mhsm := range []bool{false, true} {
-	name := "KV"
-	// if mhsm {
-	// 	name = "MHSM"
-	// }
-	t.Run(name, func(t *testing.T) {
-		client := startTest(t, false)
-		key := createRandomName(t, "testreleasekey")
-
-		// retry creating the key because Key Vault sometimes can't reach the fake
-		// attestation service we use in CI for several minutes after deployment
-		var createResp azkeys.CreateKeyResponse
-		var err error
-		for i := 0; i < 5; i++ {
-			params := azkeys.CreateKeyParameters{
-				Curve: to.Ptr(azkeys.CurveNameP256K),
-				KeyAttributes: &azkeys.KeyAttributes{
-					Exportable: to.Ptr(true),
-				},
-				Kty: to.Ptr(azkeys.KeyTypeECHSM),
-				ReleasePolicy: &azkeys.KeyReleasePolicy{
-					EncodedPolicy: getMarshalledReleasePolicy(attestationURL),
-					Immutable:     to.Ptr(true),
-				},
-			}
-			createResp, err = client.CreateKey(context.Background(), key, params, nil)
-			if err == nil {
-				break
-			}
-			if i < 4 {
-				recording.Sleep(30 * time.Second)
-			}
+	for _, mhsm := range []bool{false, true} {
+		name := "KV"
+		if mhsm {
+			name = "MHSM"
 		}
-		require.NoError(t, err)
-		require.NotNil(t, createResp.Key.KID)
-		defer cleanUpKey(t, client, createResp.Key.KID)
+		t.Run(name, func(t *testing.T) {
+			client := startTest(t, false)
+			key := createRandomName(t, "testreleasekey")
 
-		attestationClient, err := recording.NewRecordingHTTPClient(t, nil)
-		require.NoError(t, err)
-		req, err := http.NewRequest("GET", fmt.Sprintf("%s/generate-test-token", attestationURL), nil)
-		require.NoError(t, err)
-		resp, err := attestationClient.Do(req)
-		require.NoError(t, err)
-		require.Equal(t, resp.StatusCode, http.StatusOK)
-		defer resp.Body.Close()
+			// retry creating the key because Key Vault sometimes can't reach the fake
+			// attestation service we use in CI for several minutes after deployment
+			var createResp azkeys.CreateKeyResponse
+			var err error
+			for i := 0; i < 5; i++ {
+				params := azkeys.CreateKeyParameters{
+					Curve: to.Ptr(azkeys.CurveNameP256K),
+					KeyAttributes: &azkeys.KeyAttributes{
+						Exportable: to.Ptr(true),
+					},
+					Kty: to.Ptr(azkeys.KeyTypeECHSM),
+					ReleasePolicy: &azkeys.KeyReleasePolicy{
+						EncodedPolicy: getMarshalledReleasePolicy(attestationURL),
+						Immutable:     to.Ptr(true),
+					},
+				}
+				createResp, err = client.CreateKey(context.Background(), key, params, nil)
+				if err == nil {
+					break
+				}
+				if i < 4 {
+					recording.Sleep(30 * time.Second)
+				}
+			}
+			require.NoError(t, err)
+			require.NotNil(t, createResp.Key.KID)
+			defer cleanUpKey(t, client, createResp.Key.KID)
 
-		var tR struct {
-			Token *string `json:"token"`
-		}
-		err = json.NewDecoder(resp.Body).Decode(&tR)
-		require.NoError(t, err)
+			attestationClient, err := recording.NewRecordingHTTPClient(t, nil)
+			require.NoError(t, err)
+			req, err := http.NewRequest("GET", fmt.Sprintf("%s/generate-test-token", attestationURL), nil)
+			require.NoError(t, err)
+			resp, err := attestationClient.Do(req)
+			require.NoError(t, err)
+			require.Equal(t, http.StatusOK, resp.StatusCode)
+			defer resp.Body.Close()
+
+			var tR struct {
+				Token *string `json:"token"`
+			}
+			err = json.NewDecoder(resp.Body).Decode(&tR)
+			require.NoError(t, err)
+
+			params := azkeys.ReleaseParameters{TargetAttestationToken: tR.Token}
+			testSerde(t, &params)
+			releaseResp, err := client.Release(context.Background(), key, "", params, nil)
+			if err != nil && strings.Contains(err.Error(), "Target environment attestation statement cannot be verified.") {
+				t.Skip("test encountered a transient service fault; see https://github.com/Azure/azure-sdk-for-net/issues/27957")
+			}
+			require.NoError(t, err)
+			require.NotEmpty(t, releaseResp.KeyReleaseResult.Value)
+			testSerde(t, &releaseResp.KeyReleaseResult)
+		})
+	}
 
-		params := azkeys.ReleaseParameters{TargetAttestationToken: tR.Token}
-		testSerde(t, &params)
-		releaseResp, err := client.Release(context.Background(), key, "", params, nil)
-		if err != nil && strings.Contains(err.Error(), "Target environment attestation statement cannot be verified.") {
-			t.Skip("test encountered a transient service fault; see https://github.com/Azure/azure-sdk-for-net/issues/27957")
-		}
-		require.NoError(t, err)
-		require.NotEmpty(t, releaseResp.KeyReleaseResult.Value)
-		testSerde(t, &releaseResp.KeyReleaseResult)
-	})
 }
 
-//}
-
 func TestRotateKey(t *testing.T) {
 	for _, mhsm := range []bool{false, true} {
 		name := "KV"
@@ -731,3 +728,31 @@ func TestWrapUnwrap(t *testing.T) {
 		})
 	}
 }
+
+func TestAPIVersion(t *testing.T) {
+	apiVersion := "7.3"
+	var requireVersion = func(req *http.Request) bool {
+		version := req.URL.Query().Get("api-version")
+		require.Equal(t, version, apiVersion)
+		return true
+	}
+	srv, close := mock.NewServer(mock.WithTransformAllRequestsToTestServerUrl())
+	defer close()
+	srv.AppendResponse(
+		mock.WithStatusCode(200),
+		mock.WithPredicate(requireVersion),
+	)
+	srv.AppendResponse(mock.WithStatusCode(http.StatusInternalServerError))
+
+	opts := &azkeys.ClientOptions{
+		ClientOptions: azcore.ClientOptions{
+			Transport:  srv,
+			APIVersion: apiVersion,
+		},
+	}
+	client, err := azkeys.NewClient(vaultURL, &azcred.Fake{}, opts)
+	require.NoError(t, err)
+
+	_, err = client.GetKey(context.Background(), "name", "", nil)
+	require.NoError(t, err)
+}
