@@ -6,6 +6,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"maps"
 	"os"
 	"os/exec"
 	"os/signal"
@@ -29,6 +30,7 @@ import (
 	"github.com/buildkite/agent/v3/internal/awslib"
 	awssigner "github.com/buildkite/agent/v3/internal/cryptosigner/aws"
 	"github.com/buildkite/agent/v3/internal/experiments"
+	"github.com/buildkite/agent/v3/internal/job"
 	"github.com/buildkite/agent/v3/internal/job/hook"
 	"github.com/buildkite/agent/v3/internal/osutil"
 	"github.com/buildkite/agent/v3/internal/shell"
@@ -41,7 +43,6 @@ import (
 	"github.com/buildkite/shellwords"
 	"github.com/lestrrat-go/jwx/v2/jwk"
 	"github.com/urfave/cli"
-	"golang.org/x/exp/maps"
 )
 
 const startDescription = `Usage:
@@ -62,6 +63,8 @@ Example:
     $ buildkite-agent start --token xxx`
 
 var (
+	minGracePeriod = 10
+
 	verificationFailureBehaviors = []string{agent.VerificationBehaviourBlock, agent.VerificationBehaviourWarn}
 
 	buildkiteSetEnvironmentVariables = []*regexp.Regexp{
@@ -162,11 +165,14 @@ type AgentStartConfig struct {
 
 	HealthCheckAddr string `cli:"health-check-addr"`
 
+	// Datadog statsd metrics config
 	MetricsDatadog              bool   `cli:"metrics-datadog"`
 	MetricsDatadogHost          string `cli:"metrics-datadog-host"`
 	MetricsDatadogDistributions bool   `cli:"metrics-datadog-distributions"`
-	TracingBackend              string `cli:"tracing-backend"`
-	TracingServiceName          string `cli:"tracing-service-name"`
+
+	// Tracing config
+	TracingBackend     string `cli:"tracing-backend"`
+	TracingServiceName string `cli:"tracing-service-name"`
 
 	// Global flags
 	Debug                     bool     `cli:"debug"`
@@ -236,6 +242,10 @@ func (asc AgentStartConfig) Features(ctx context.Context) []string {
 		features = append(features, "no-script-eval")
 	}
 
+	if asc.NoHTTP2 {
+		features = append(features, "no-http2")
+	}
+
 	if len(asc.AllowedRepositories) > 0 {
 		features = append(features, "allowed-repositories")
 	}
@@ -248,6 +258,18 @@ func (asc AgentStartConfig) Features(ctx context.Context) []string {
 		features = append(features, fmt.Sprintf("experiment-%s", exp))
 	}
 
+	if envHasKey("HTTP_PROXY") || envHasKey("http_proxy") {
+		features = append(features, "env-http-proxy")
+	}
+
+	if envHasKey("GODEBUG") {
+		features = append(features, "env-godebug")
+	}
+
+	if asc.MetricsDatadog {
+		features = append(features, "datadog-metrics")
+	}
+
 	return features
 }
 
@@ -766,6 +788,9 @@ var AgentStartCommand = cli.Command{
 		))
 		defer done()
 
+		// used later to force the shutdown of the agent
+		ctx, cancel := context.WithCancel(ctx)
+
 		// Verify that the bootstrap buildkite-agent executable matches the current host agent
 		// executable. In development builds, it exits on mismatch; otherwise it only logs a warning.
 		// This is to avoid confusion when making changes to the buildkite-agent but forgetting to
@@ -901,7 +926,7 @@ var AgentStartCommand = cli.Command{
 			return fmt.Errorf(
 				"the given tracing backend %q is not supported. Valid backends are: %q",
 				cfg.TracingBackend,
-				maps.Keys(tracetools.ValidTracingBackends),
+				slices.Collect(maps.Keys(tracetools.ValidTracingBackends)),
 			)
 		}
 
@@ -1255,7 +1280,7 @@ var AgentStartCommand = cli.Command{
 		}
 
 		// Handle process signals
-		signals := handlePoolSignals(ctx, l, pool)
+		signals := handlePoolSignals(ctx, l, pool, cancel, cfg.CancelGracePeriod)
 		defer close(signals)
 
 		l.Info("Starting %d Agent(s)", cfg.Spawn)
@@ -1343,7 +1368,7 @@ func parseAndValidateJWKS(ctx context.Context, keysetType, path string) (jwk.Set
 	return jwks, nil
 }
 
-func handlePoolSignals(ctx context.Context, l logger.Logger, pool *agent.AgentPool) chan os.Signal {
+func handlePoolSignals(ctx context.Context, l logger.Logger, pool *agent.AgentPool, cancel context.CancelFunc, cancelGracePeriod int) chan os.Signal {
 	signals := make(chan os.Signal, 1)
 	signal.Notify(signals, os.Interrupt,
 		syscall.SIGHUP,
@@ -1373,8 +1398,30 @@ func handlePoolSignals(ctx context.Context, l logger.Logger, pool *agent.AgentPo
 					l.Info("Received CTRL-C, send again to forcefully kill the agent(s)")
 					pool.Stop(true)
 				} else {
-					l.Info("Forcefully stopping running jobs and stopping the agent(s)")
-					pool.Stop(false)
+					l.Info("Forcefully stopping running jobs and stopping the agent(s) in %d seconds", cancelGracePeriod)
+
+					gracefulContext, _ := job.WithGracePeriod(ctx, time.Duration(max(cancelGracePeriod, minGracePeriod))*time.Second)
+
+					go func() {
+						l.Info("Forced agent(s) to stop")
+						pool.Stop(false) // one last chance to stop
+
+						// Wait half the grace period before cancelling the context
+						time.Sleep(time.Duration(max(cancelGracePeriod/2, minGracePeriod/2)) * time.Second)
+
+						l.Info("Cancelling all internal tasks and API requests")
+						cancel() // cancel the context to stop all network operations
+					}()
+
+					// Once pending retries and requests are cancelled,
+					// the main goroutine should exit before this grace period expires, ending the program.
+					// If that doesn't happen, exit 1 below.
+					<-gracefulContext.Done()
+					l.Info("exiting with status 1")
+
+					// this should only be called if the context cancellation and forceful stop fails
+					os.Exit(1)
+
 				}
 			default:
 				l.Debug("Ignoring signal `%s`", sig.String())
@@ -1541,3 +1588,9 @@ func leaderPinger(ctx context.Context, l logger.Logger, path, leaderPath string)
 		}
 	}
 }
+
+func envHasKey(key string) bool {
+	_, ok := os.LookupEnv(key)
+	return ok
+
+}
