@@ -2,8 +2,13 @@ package signature
 
 import (
 	"context"
+	"crypto"
+	"crypto/ecdsa"
+	"crypto/x509"
+	"encoding/pem"
 	"errors"
 	"fmt"
+	"io"
 	"math/rand"
 	"os"
 	"path"
@@ -134,6 +139,133 @@ func TestSignVerify(t *testing.T) {
 	}
 }
 
+var _ crypto.Signer = testECDSASigner{}
+
+type testECDSASigner struct {
+	privateKey crypto.PrivateKey
+	publickKey crypto.PublicKey
+}
+
+func (m testECDSASigner) Public() crypto.PublicKey { return m.publickKey }
+
+func (m testECDSASigner) Sign(rand io.Reader, digest []byte, opts crypto.SignerOpts) ([]byte, error) {
+	return ecdsa.SignASN1(rand, m.privateKey.(*ecdsa.PrivateKey), digest)
+}
+
+func (m testECDSASigner) Algorithm() jwa.KeyAlgorithm {
+	return jwa.ES256
+}
+
+func TestSignVerifyCryptoSigner(t *testing.T) {
+
+	t.Parallel()
+	ctx := context.Background()
+
+	step := &pipeline.CommandStep{
+		Command: "llamas",
+		Plugins: pipeline.Plugins{
+			{
+				Source: "some-plugin#v1.0.0",
+				Config: nil,
+			},
+			{
+				Source: "another-plugin#v3.4.5",
+				Config: map[string]any{"llama": "Kuzco"},
+			},
+		},
+		Env: map[string]string{
+			"CONTEXT": "cats",
+			"DEPLOY":  "0",
+		},
+	}
+	// The pipeline-level env that the agent uploads:
+	signEnv := map[string]string{
+		"DEPLOY": "1",
+	}
+
+	// The backend combines the pipeline and step envs, providing a new env:
+	verifyEnv := map[string]string{
+		"CONTEXT": "cats",
+		"DEPLOY":  "0",
+		"MISC":    "llama drama",
+	}
+
+	stepWithInvariants := &CommandStepWithInvariants{
+		CommandStep:   *step,
+		RepositoryURL: "fake-repo",
+	}
+
+	cases := []struct {
+		name              string
+		alg               jwa.SignatureAlgorithm
+		expectedSignature string
+	}{
+		{
+			name:              "should sign using crypto.Signer",
+			alg:               jwa.ES256,
+			expectedSignature: "eyJhbGciOiJFUzI1NiJ9..Op5KSww95n5s1b9jz0Me5UGqUQPcHzEIFvkWTB_yEv6qEDnnFUO1XsC5592fQoAcB0VnPnHaK31iSiCypREIdA",
+		},
+	}
+
+	wd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("os.Getwd() error = %v", err)
+	}
+
+	privateKeyPath := path.Join(wd, "fixtures", "crypto_signer", "P256", "private.pem")
+	pemPrivateKey, err := os.ReadFile(privateKeyPath)
+	if err != nil {
+		t.Fatalf("os.ReadFile(%q) error = %v", privateKeyPath, err)
+	}
+
+	block, _ := pem.Decode([]byte(pemPrivateKey))
+	x509Encoded := block.Bytes
+	privateKey, err := x509.ParseECPrivateKey(x509Encoded)
+	if err != nil {
+		t.Fatalf("x509.ParseECPrivateKey(%v) error = %v", x509Encoded, err)
+	}
+
+	publicKeyPath := path.Join(wd, "fixtures", "crypto_signer", "P256", "public.pem")
+	pemPublicKey, err := os.ReadFile(publicKeyPath)
+	if err != nil {
+		t.Fatalf("os.ReadFile(%q) error = %v", publicKeyPath, err)
+	}
+
+	blockPub, _ := pem.Decode([]byte(pemPublicKey))
+	x509EncodedPub := blockPub.Bytes
+	genericPublicKey, err := x509.ParsePKIXPublicKey(x509EncodedPub)
+	if err != nil {
+		t.Fatalf("x509.ParsePKIXPublicKey(%v) error = %v", x509EncodedPub, err)
+	}
+
+	publicKey := genericPublicKey.(*ecdsa.PublicKey)
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			t.Parallel()
+
+			sKey := testECDSASigner{
+				privateKey: privateKey,
+				publickKey: publicKey,
+			}
+
+			sig, err := Sign(ctx, sKey, stepWithInvariants, WithEnv(signEnv))
+			if err != nil {
+				t.Fatalf("Sign(ctx, sKey, %v, WithEnv(%v)) error = %v", stepWithInvariants, signEnv, err)
+			}
+
+			if sig.Algorithm != tc.alg.String() {
+				t.Errorf("Signature.Algorithm = %v, want %v", sig.Algorithm, tc.alg)
+			}
+
+			if err := Verify(ctx, sig, sKey, stepWithInvariants, WithEnv(verifyEnv)); err != nil {
+				t.Errorf("Verify(ctx, %v, verifier, %v, WithEnv(%v)) = %v", sig, stepWithInvariants, verifyEnv, err)
+			}
+
+		})
+	}
+}
+
 type testFields map[string]any
 
 func (m testFields) SignedFields() (map[string]any, error) { return m, nil }
@@ -384,6 +516,130 @@ func TestSignVerifyEnv(t *testing.T) {
 	}
 }
 
+func TestSignVerify_NilVsEmpty(t *testing.T) {
+	t.Parallel()
+	ctx := context.Background()
+
+	cases := []struct {
+		name       string
+		stepSign   *pipeline.CommandStep
+		stepVerify *pipeline.CommandStep
+	}{
+		{
+			name: "env both non-empty",
+			stepSign: &pipeline.CommandStep{
+				Command: "llamas",
+				Env: map[string]string{
+					"CONTEXT": "cats",
+					"DEPLOY":  "0",
+				},
+			},
+			stepVerify: &pipeline.CommandStep{
+				Command: "llamas",
+				Env: map[string]string{
+					"CONTEXT": "cats",
+					"DEPLOY":  "0",
+				},
+			},
+		},
+		{
+			name:       "env sign nil verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Env: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Env: nil},
+		},
+		{
+			name:       "env sign empty verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Env: map[string]string{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Env: nil},
+		},
+		{
+			name:       "env sign nil verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Env: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Env: map[string]string{}},
+		},
+		{
+			name:       "env sign empty verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Env: map[string]string{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Env: map[string]string{}},
+		},
+		{
+			name:       "plugins sign nil verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Plugins: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Plugins: nil},
+		},
+		{
+			name:       "plugins sign nil verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Plugins: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Plugins: pipeline.Plugins{}},
+		},
+		{
+			name:       "plugins sign empty verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Plugins: pipeline.Plugins{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Plugins: nil},
+		},
+		{
+			name:       "plugins sign empty verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Plugins: pipeline.Plugins{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Plugins: pipeline.Plugins{}},
+		},
+		{
+			name:       "matrix sign nil verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Matrix: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Matrix: nil},
+		},
+		{
+			name:       "matrix sign nil verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Matrix: nil},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Matrix: &pipeline.Matrix{}},
+		},
+		{
+			name:       "matrix sign empty verify nil",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Matrix: &pipeline.Matrix{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Matrix: nil},
+		},
+		{
+			name:       "matrix sign empty verify empty",
+			stepSign:   &pipeline.CommandStep{Command: "llamas", Matrix: &pipeline.Matrix{}},
+			stepVerify: &pipeline.CommandStep{Command: "llamas", Matrix: &pipeline.Matrix{}},
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			t.Parallel()
+
+			keyStr, keyAlg := "alpacas", jwa.HS256
+			signer, verifier, err := jwkutil.NewSymmetricKeyPairFromString(keyID, keyStr, keyAlg)
+			if err != nil {
+				t.Fatalf("jwkutil.NewSymmetricKeyPairFromString(%q, %q, %q) error = %v", keyID, keyStr, keyAlg, err)
+			}
+
+			key, ok := signer.Key(0)
+			if !ok {
+				t.Fatalf("signer.Key(0) = _, false, want true")
+			}
+
+			toSign := &CommandStepWithInvariants{
+				CommandStep:   *tc.stepSign,
+				RepositoryURL: fakeRepositoryURL,
+			}
+			toVerify := &CommandStepWithInvariants{
+				CommandStep:   *tc.stepVerify,
+				RepositoryURL: fakeRepositoryURL,
+			}
+
+			sig, err := Sign(ctx, key, toSign)
+			if err != nil {
+				t.Fatalf("Sign(ctx, key, %v) error = %v", toSign, err)
+			}
+
+			if err := Verify(ctx, sig, verifier, toVerify); err != nil {
+				t.Errorf("Verify(ctx, %v, verifier, %v) = %v", sig, toVerify, err)
+			}
+		})
+	}
+}
+
 func TestSignatureStability(t *testing.T) {
 	t.Parallel()
 	ctx := context.Background()
